
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

from __future__ import absolute_import
import json

version_json = '''
{
 "date": "2019-08-15T02:58:37+0100",
 "dirty": false,
 "error": null,
 "full-revisionid": "eda138b001a07f5afb2a971db2fe2f6baba602a6",
 "version": "2019.8.16"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
